'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extend2 = require('lodash/extend');

var _extend3 = _interopRequireDefault(_extend2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _createReactClass = require('create-react-class');

var _createReactClass2 = _interopRequireDefault(_createReactClass);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _loreReactFormsBootstrap = require('lore-react-forms-bootstrap');

var _Step = require('./forms/Step');

var _Step2 = _interopRequireDefault(_Step);

var _Confirmation = require('./forms/Confirmation');

var _Confirmation2 = _interopRequireDefault(_Confirmation);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var formMap = {
  step: {
    render: function render(props, config) {
      return _react2.default.createElement(_Step2.default, _extends({}, props, config));
    }
  },
  confirmation: {
    render: function render(props, config) {
      return _react2.default.createElement(_Confirmation2.default, _extends({}, props, config));
    }
  }
};

exports.default = (0, _createReactClass2.default)({
  displayName: 'Wizard',

  propTypes: {
    steps: _propTypes2.default.array.isRequired,
    modelName: _propTypes2.default.string.isRequired
  },

  getInitialState: function getInitialState() {
    var data = this.props.data;


    return {
      stepIndex: 0,
      data: data || {},
      isSaving: false,
      request: null,
      hasError: false
    };
  },


  request: function request(data) {
    var modelName = this.props.modelName;

    return lore.actions[modelName].create(data).payload;
  },

  onSubmit: function onSubmit(newData) {
    var data = this.state.data;

    var nextData = (0, _extend3.default)({}, data, newData);

    this.setState({
      isSaving: true,
      hasError: false,
      data: nextData,
      request: this.request(nextData)
    });
  },

  onRequestSuccess: function onRequestSuccess(request) {
    var steps = this.props.steps;

    var lastStepIndex = steps.length - 1;

    this.setState({
      isSaving: false,
      request: request,
      hasError: false,
      stepIndex: lastStepIndex
    });
  },

  onRequestError: function onRequestError(request) {
    this.setState({
      isSaving: false,
      request: request,
      hasError: true
    });
  },

  onResetWizard: function onResetWizard() {
    this.setState((0, _extend3.default)(this.getInitialState(), {
      stepIndex: 0
    }));
  },

  onNext: function onNext(newData, newStepIndex) {
    var _state = this.state,
        data = _state.data,
        stepIndex = _state.stepIndex;


    this.setState({
      stepIndex: newStepIndex || stepIndex + 1,
      data: (0, _extend3.default)({}, data, newData)
    });
  },

  onPrevious: function onPrevious(newData, newStepIndex) {
    var _state2 = this.state,
        data = _state2.data,
        stepIndex = _state2.stepIndex;


    this.setState({
      stepIndex: newStepIndex || stepIndex - 1,
      data: (0, _extend3.default)({}, data, newData)
    });
  },

  onChangeStep: function onChangeStep(newData, newStepIndex) {
    var data = this.state.data;


    this.setState({
      stepIndex: newStepIndex,
      data: (0, _extend3.default)({}, data, newData)
    });
  },


  render: function render() {
    var _this = this;

    var _props = this.props,
        config = _props.config,
        steps = _props.steps,
        modelName = _props.modelName;
    var _state3 = this.state,
        data = _state3.data,
        stepIndex = _state3.stepIndex,
        request = _state3.request,
        isSaving = _state3.isSaving,
        hasError = _state3.hasError;
    var _lore$config$forms = lore.config.forms,
        schemas = _lore$config$forms.schemas,
        fieldMap = _lore$config$forms.fieldMap,
        actionMap = _lore$config$forms.actionMap;


    var stepConfig = steps[stepIndex];
    var form = formMap[stepConfig.form];

    var step = form.render({
      data: data,
      schema: schemas.default,
      fieldMap: fieldMap,
      actionMap: actionMap,
      hasError: hasError,
      request: request,
      callbacks: {
        onNext: function onNext(newData, nextStep) {
          _this.onNext(newData, nextStep);
        },
        onPrevious: function onPrevious(newData, nextStep) {
          _this.onPrevious(newData, nextStep);
        },
        onSubmit: function onSubmit(newData) {
          _this.onSubmit(newData);
        },
        onResetWizard: this.onResetWizard
      }
    }, stepConfig);

    return _react2.default.createElement(
      _loreReactFormsBootstrap.Overlay,
      { isVisible: isSaving },
      _react2.default.createElement(
        'div',
        null,
        request && isSaving ? _react2.default.createElement(_loreReactFormsBootstrap.Request, {
          request: request,
          reducer: modelName,
          onSuccess: this.onRequestSuccess,
          onError: this.onRequestError
        }) : null,
        step
      )
    );
  }

});