'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _createReactClass = require('create-react-class');

var _createReactClass2 = _interopRequireDefault(_createReactClass);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _loreUtils = require('lore-utils');

var _loreReactForms = require('lore-react-forms');

var _loreReactFormsBootstrap = require('lore-react-forms-bootstrap');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = (0, _createReactClass2.default)({
  displayName: 'Create.Overlay',

  propTypes: {
    data: _propTypes2.default.object,
    validators: _propTypes2.default.object,
    onChange: _propTypes2.default.func,
    onSubmit: _propTypes2.default.func,
    callbacks: _propTypes2.default.object,
    schema: _propTypes2.default.object.isRequired,
    fieldMap: _propTypes2.default.object.isRequired,
    actionMap: _propTypes2.default.object.isRequired,
    config: _propTypes2.default.object.isRequired,
    modelName: _propTypes2.default.string.isRequired
  },

  getInitialState: function getInitialState() {
    var data = this.props.data;


    return {
      key: 0,
      data: data || {},
      isSaving: false,
      showSuccessMessage: false,
      hasError: false,
      request: null
    };
  },

  request: function request(data) {
    var modelName = this.props.modelName;

    return lore.actions[modelName].create(data).payload;
  },

  onSubmit: function onSubmit() {
    var data = this.state.data;


    this.setState({
      isSaving: true,
      showSuccessMessage: false,
      hasError: false,
      request: this.request(data, this)
    });
  },

  onRequestSuccess: function onRequestSuccess(request) {
    this.setState({
      isSaving: false,
      request: request,
      showSuccessMessage: true,
      hasError: false,
      key: this.state.key + 1,
      data: this.getInitialState().data
    });
  },

  onRequestError: function onRequestError(request) {
    this.setState({
      isSaving: false,
      request: request,
      hasError: true
    });
  },

  onChange: function onChange(name, value) {
    var nextData = _lodash2.default.merge({}, this.state.data);
    nextData[name] = value;
    this.setState({
      data: nextData
    });
  },

  getValidators: function getValidators(data) {
    var validators = this.props.validators;

    return (0, _loreUtils.result)(validators || {}, data);
  },

  render: function render() {
    var _props = this.props,
        schema = _props.schema,
        fieldMap = _props.fieldMap,
        actionMap = _props.actionMap,
        config = _props.config,
        modelName = _props.modelName;
    var _state = this.state,
        key = _state.key,
        data = _state.data,
        request = _state.request,
        isSaving = _state.isSaving,
        showSuccessMessage = _state.showSuccessMessage,
        hasError = _state.hasError;


    var validators = this.getValidators(data);

    var requestProps = {
      request: request,
      reducer: modelName,
      onSuccess: this.onRequestSuccess,
      onError: this.onRequestError
    };

    return _react2.default.createElement(
      _loreReactFormsBootstrap.Overlay,
      { isVisible: isSaving },
      _react2.default.createElement(
        'div',
        null,
        _react2.default.createElement(
          'div',
          { style: { padding: '20px', fontSize: '20px', fontWeight: '500' } },
          'Create ' + _lodash2.default.capitalize(modelName)
        ),
        request && isSaving ? _react2.default.createElement(_loreReactFormsBootstrap.Request, requestProps) : null,
        showSuccessMessage ? _react2.default.createElement(
          'div',
          { style: { padding: '0px 20px' } },
          _react2.default.createElement(_loreReactFormsBootstrap.SuccessMessage, {
            title: 'Success!',
            message: _lodash2.default.capitalize(modelName) + ' created.'
          })
        ) : null,
        hasError ? _react2.default.createElement(
          'div',
          { style: { padding: '0px 20px' } },
          _react2.default.createElement(
            _loreReactFormsBootstrap.RequestError,
            { request: request },
            function (request) {
              return request.error;
            }
          )
        ) : null,
        _react2.default.createElement(_loreReactForms.SchemaForm, {
          key: key,
          data: data,
          validators: validators,
          onChange: this.onChange,
          onSubmit: this.onSubmit,
          callbacks: {
            onSubmit: this.onSubmit,
            onCancel: this.props.onCancel
          },
          schema: schema,
          fieldMap: fieldMap,
          actionMap: actionMap,
          config: _lodash2.default.merge({
            fields: {},
            actions: []
          }, config)
        })
      )
    );
  }

});
module.exports = exports['default'];